#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <pthread.h>
#include <malloc.h>

extern void  __real_free(void *ptr);
extern void *__real_malloc(size_t size);
extern void *__real_calloc(size_t nelem, size_t elsize);
extern void *__real_realloc(void *ptr, size_t size);
extern void *__real_memalign(size_t alignment, size_t size);

static long consumed_memory = 0;

//Adopted from libscm to support 64-bit values
static inline void atomic_long_add(volatile long *atomic, long val) {
    __asm__ __volatile__("lock; addq %1,%0"
                     : "=m" (*atomic)
                                 : "ir" (val), "m" (*atomic));
}

void print_memory_consumption()
{
    struct mallinfo info = mallinfo();
    printf("%ld\t%d\t%ld\n", consumed_memory, info.uordblks, (long)info.uordblks - consumed_memory);
}

void __wrap_free(void *ptr)
{
    atomic_long_add(&consumed_memory, -malloc_usable_size(ptr));
    __real_free(ptr);
}


void *__wrap_malloc(size_t size)
{
    void *ptr = __real_malloc(size);
    atomic_long_add(&consumed_memory, malloc_usable_size(ptr));
    return ptr;
}

void *__wrap_calloc(size_t nelem, size_t elsize)
{
    void *ptr = __real_calloc(nelem, elsize);
    atomic_long_add(&consumed_memory, malloc_usable_size(ptr));
    return ptr;
}

void *__wrap_realloc(void *ptr, size_t size)
{
    atomic_long_add(&consumed_memory, -malloc_usable_size(ptr));
    void *new_ptr = __real_realloc(ptr, size);
    atomic_long_add(&consumed_memory, malloc_usable_size(new_ptr));
    return new_ptr;
}

void *__wrap_memalign(size_t alignment, size_t size)
{
    void *ptr = __real_memalign(alignment, size);
    atomic_long_add(&consumed_memory, malloc_usable_size(ptr));
    return ptr;
}

